@@ -28,6 +28,7 @@
 
 #include <ell/ell.h>
 
+#include "ell/useful.h"
 #include "src/util.h"
 #include "src/crypto.h"
 #include "src/ie.h"
@@ -726,17 +727,17 @@ static int parse_ciphers(const uint8_t *data, size_t len,
 	if (len < 2)
 		return -EBADMSG;
 
-	out_info->preauthentication = util_is_bit_set(data[0], 0);
-	out_info->no_pairwise = util_is_bit_set(data[0], 1);
-	out_info->ptksa_replay_counter = util_bit_field(data[0], 2, 2);
-	out_info->gtksa_replay_counter = util_bit_field(data[0], 4, 2);
-	out_info->mfpr = util_is_bit_set(data[0], 6);
-	out_info->mfpc = util_is_bit_set(data[0], 7);
-	out_info->peerkey_enabled = util_is_bit_set(data[1], 1);
-	out_info->spp_a_msdu_capable = util_is_bit_set(data[1], 2);
-	out_info->spp_a_msdu_required = util_is_bit_set(data[1], 3);
-	out_info->pbac = util_is_bit_set(data[1], 4);
-	out_info->extended_key_id = util_is_bit_set(data[1], 5);
+	out_info->preauthentication = test_bit(data, 0);
+	out_info->no_pairwise = test_bit(data, 1);
+	out_info->ptksa_replay_counter = bit_field(data[0], 2, 2);
+	out_info->gtksa_replay_counter = bit_field(data[0], 4, 2);
+	out_info->mfpr = test_bit(data, 6);
+	out_info->mfpc = test_bit(data, 7);
+	out_info->peerkey_enabled = test_bit(data + 1, 1);
+	out_info->spp_a_msdu_capable = test_bit(data + 1, 2);
+	out_info->spp_a_msdu_required = test_bit(data + 1, 3);
+	out_info->pbac = test_bit(data + 1, 4);
+	out_info->extended_key_id = test_bit(data + 1, 5);
 
 	/*
 	 * BIP—default group management cipher suite in an RSNA with
@@ -1175,6 +1176,33 @@ bool ie_build_rsne(const struct ie_rsn_info *info, uint8_t *to)
 	return true;
 }
 
+bool ie_rsne_is_wpa3_personal(const struct ie_rsn_info *info)
+{
+	bool is_transition = info->akm_suites & IE_RSN_AKM_SUITE_PSK;
+	/*
+	 * WPA3 Specification, Version 2
+	 *
+	 * Section 2.2 WPA3-Personal only Mode:
+	 * 1. An AP shall enable at least AKM suite selector 00-0F-AC:8 in
+	 * the BSS
+	 * 3.  An AP shall not enable AKM suite selector: 00-0F-AC:2, 00-0F-AC:6
+	 * 5. an AP shall set MFPC to 1, MFPR to 1
+	 *
+	 * Section 2.3 WPA3-Personal transition Mode:
+	 * 1. an AP shall enable at least AKM suite selectors 00-0F-AC:2 and
+	 * 00-0F-AC:8 in the BSS
+	 * 3. an AP should enable AKM suite selector: 00-0F-AC:6
+	 * 5. an AP shall set MFPC to 1, MFPR to 0
+	 */
+	if (!(info->akm_suites & IE_RSN_AKM_SUITE_SAE_SHA256))
+		return false;
+
+	if (!info->mfpc)
+		return false;
+
+	return is_transition || info->mfpr;
+}
+
 bool ie_build_osen(const struct ie_rsn_info *info, uint8_t *to)
 {
 	unsigned int pos;
@@ -1422,10 +1450,10 @@ int ie_parse_wpa(struct ie_tlv_iter *iter, struct ie_rsn_info *out_info)
 	if (len < 2)
 		return -EBADMSG;
 
-	out_info->preauthentication = util_is_bit_set(data[0], 0);
-	out_info->no_pairwise = util_is_bit_set(data[0], 1);
-	out_info->ptksa_replay_counter = util_bit_field(data[0], 2, 2);
-	out_info->gtksa_replay_counter = util_bit_field(data[0], 4, 2);
+	out_info->preauthentication = test_bit(data, 0);
+	out_info->no_pairwise = test_bit(data, 1);
+	out_info->ptksa_replay_counter = bit_field(data[0], 2, 2);
+	out_info->gtksa_replay_counter = bit_field(data[0], 4, 2);
 
 	RSNE_ADVANCE(data, len, 2);
 
@@ -1823,9 +1851,9 @@ static int ie_parse_ht_capability(struct ie_tlv_iter *iter, int32_t rssi,
 	/* Parse out channel width set and short GI */
 	ht_cap = l_get_u8(data++);
 
-	support_40mhz = util_is_bit_set(ht_cap, 1);
-	short_gi_20mhz = util_is_bit_set(ht_cap, 5);
-	short_gi_40mhz = util_is_bit_set(ht_cap, 6);
+	support_40mhz = test_bit(&ht_cap, 1);
+	short_gi_20mhz = test_bit(&ht_cap, 5);
+	short_gi_40mhz = test_bit(&ht_cap, 6);
 
 	data += 2;
 
@@ -1840,10 +1868,8 @@ static int ie_parse_ht_capability(struct ie_tlv_iter *iter, int32_t rssi,
 	 */
 	for (i = 31; i >= 0; i--) {
 		uint64_t drate;
-		uint8_t byte = i / 8;
-		uint8_t bit = i % 8;
 
-		if (!util_is_bit_set(data[byte], bit))
+		if (!test_bit(data, i))
 			continue;
 
 		if (!support_40mhz)
@@ -1951,8 +1977,8 @@ static int ie_parse_vht_capability(struct ie_tlv_iter *vht_iter,
 
 	ht_cap = l_get_u8(data);
 
-	short_gi_20mhz = util_is_bit_set(ht_cap, 5);
-	short_gi_40mhz = util_is_bit_set(ht_cap, 6);
+	short_gi_20mhz = test_bit(&ht_cap, 5);
+	short_gi_40mhz = test_bit(&ht_cap, 6);
 
 	/* now move onto VHT */
 	len = ie_tlv_iter_get_length(vht_iter);
@@ -1962,9 +1988,9 @@ static int ie_parse_vht_capability(struct ie_tlv_iter *vht_iter,
 
 	data = ie_tlv_iter_get_data(vht_iter);
 
-	channel_width_set = util_bit_field(*data, 2, 2);
-	short_gi_80mhz = util_bit_field(*data, 5, 1);
-	short_gi_160mhz = util_bit_field(*data, 6, 1);
+	channel_width_set = bit_field(*data, 2, 2);
+	short_gi_80mhz = bit_field(*data, 5, 1);
+	short_gi_160mhz = bit_field(*data, 6, 1);
 
 	data += 4;
 
@@ -1978,9 +2004,9 @@ static int ie_parse_vht_capability(struct ie_tlv_iter *vht_iter,
 
 	/* NSS->MCS map values are grouped in 2-bit values */
 	for (mcs = 15; mcs >= 0; mcs -= 2) {
-		uint8_t rx_val = util_bit_field(rx_mcs_map[mcs / 8],
+		uint8_t rx_val = bit_field(rx_mcs_map[mcs / 8],
 							mcs % 8, 2);
-		uint8_t tx_val = util_bit_field(tx_mcs_map[mcs / 8],
+		uint8_t tx_val = bit_field(tx_mcs_map[mcs / 8],
 							mcs % 8, 2);
 
 		/*
@@ -2255,7 +2281,7 @@ int ie_parse_fast_bss_transition(struct ie_tlv_iter *iter, uint32_t mic_len,
 			if (subelem_len < 35 || subelem_len > 51)
 				return -EINVAL;
 
-			info->gtk_key_id = util_bit_field(data[0], 0, 2);
+			info->gtk_key_id = bit_field(data[0], 0, 2);
 			info->gtk_len = data[2];
 
 			/*
@@ -2390,17 +2416,17 @@ int ie_parse_neighbor_report(struct ie_tlv_iter *iter,
 
 	memcpy(info->addr, data + 0, 6);
 
-	info->ht = util_is_bit_set(data[8], 3);
-	info->md = util_is_bit_set(data[8], 2);
-	info->immediate_block_ack = util_is_bit_set(data[8], 1);
-	info->delayed_block_ack = util_is_bit_set(data[8], 0);
-	info->rm = util_is_bit_set(data[9], 7);
-	info->apsd = util_is_bit_set(data[9], 6);
-	info->qos = util_is_bit_set(data[9], 5);
-	info->spectrum_mgmt = util_is_bit_set(data[9], 4);
-	info->key_scope = util_is_bit_set(data[9], 3);
-	info->security = util_is_bit_set(data[9], 2);
-	info->reachable = util_bit_field(data[9], 0, 2);
+	info->ht = test_bit(data + 8, 3);
+	info->md = test_bit(data + 8, 2);
+	info->immediate_block_ack = test_bit(data + 8, 1);
+	info->delayed_block_ack = test_bit(data + 8, 0);
+	info->rm = test_bit(data + 9, 7);
+	info->apsd = test_bit(data + 9, 6);
+	info->qos = test_bit(data + 9, 5);
+	info->spectrum_mgmt = test_bit(data + 9, 4);
+	info->key_scope = test_bit(data + 9, 3);
+	info->security = test_bit(data + 9, 2);
+	info->reachable = bit_field(data[9], 0, 2);
 
 	info->oper_class = data[10];
 
@@ -2441,8 +2467,8 @@ int ie_parse_roaming_consortium(struct ie_tlv_iter *iter, size_t *num_anqp_out,
 		return -EINVAL;
 
 	num_anqp = l_get_u8(data);
-	oi1_len = util_bit_field(l_get_u8(data + 1), 0, 4);
-	oi2_len = util_bit_field(l_get_u8(data + 1), 4, 4);
+	oi1_len = bit_field(l_get_u8(data + 1), 0, 4);
+	oi2_len = bit_field(l_get_u8(data + 1), 4, 4);
 	oi3_len = len - (2 + oi1_len + oi2_len);
 
 	if (!oi1_len)
@@ -2533,8 +2559,8 @@ int ie_parse_hs20_indication(struct ie_tlv_iter *iter, uint8_t *version_out,
 
 	hs20_config = l_get_u8(data + 4);
 
-	pps_mo_present = util_is_bit_set(hs20_config, 1);
-	domain_id_present = util_is_bit_set(hs20_config, 2);
+	pps_mo_present = test_bit(&hs20_config, 1);
+	domain_id_present = test_bit(&hs20_config, 2);
 
 	/*
 	 * Hotspot 2.0 Spec - Section 3.1.1
@@ -2547,7 +2573,7 @@ int ie_parse_hs20_indication(struct ie_tlv_iter *iter, uint8_t *version_out,
 		return -EPROTOTYPE;
 
 	if (version_out)
-		*version_out = util_bit_field(hs20_config, 4, 4);
+		*version_out = bit_field(hs20_config, 4, 4);
 
 	if (pps_mo_id_out)
 		*pps_mo_id_out = 0;
