@@ -39,6 +39,7 @@
 
 #include <ell/ell.h>
 
+#include "ell/useful.h"
 #include "src/iwd.h"
 #include "src/module.h"
 #include "src/netdev.h"
@@ -64,6 +65,8 @@ struct netconfig {
 	void *user_data;
 
 	struct resolve *resolve;
+
+	struct l_acd *acd;
 };
 
 static struct l_netlink *rtnl;
@@ -606,7 +609,7 @@ static bool netconfig_ipv4_routes_install(struct netconfig *netconfig)
 	L_AUTO_FREE_VAR(char *, gateway) = NULL;
 	struct in_addr in_addr;
 	char *network;
-	char ip[INET6_ADDRSTRLEN];
+	char ip[INET_ADDRSTRLEN];
 	unsigned int prefix_len =
 		l_rtnl_address_get_prefix_length(netconfig->v4_address);
 
@@ -698,6 +701,7 @@ static void netconfig_ipv6_ifaddr_add_cmd_cb(int error, uint16_t type,
 	}
 
 	netconfig_set_dns(netconfig);
+	netconfig_set_domains(netconfig);
 }
 
 static void netconfig_ifaddr_del_cmd_cb(int error, uint16_t type,
@@ -787,10 +791,12 @@ static void netconfig_dhcp6_event_handler(struct l_dhcp6_client *client,
 	case L_DHCP6_CLIENT_EVENT_LEASE_OBTAINED:
 	case L_DHCP6_CLIENT_EVENT_LEASE_RENEWED:
 		netconfig_set_dns(netconfig);
+		netconfig_set_domains(netconfig);
 		break;
 	case L_DHCP6_CLIENT_EVENT_LEASE_EXPIRED:
 		l_debug("Lease for interface %u expired", netconfig->ifindex);
 		netconfig_set_dns(netconfig);
+		netconfig_set_domains(netconfig);
 
 		/* Fall through */
 	case L_DHCP6_CLIENT_EVENT_NO_LEASE:
@@ -801,16 +807,88 @@ static void netconfig_dhcp6_event_handler(struct l_dhcp6_client *client,
 	}
 }
 
-static void netconfig_ipv4_select_and_install(struct netconfig *netconfig)
+static void netconfig_reset_v4(struct netconfig *netconfig)
 {
-	netconfig->v4_address = netconfig_get_static4_address(netconfig);
-	if (netconfig->v4_address) {
-		netconfig->rtm_protocol = RTPROT_STATIC;
+	if (netconfig->rtm_protocol) {
+		if (netconfig->v4_address) {
+			L_WARN_ON(!l_rtnl_ifaddr_delete(rtnl,
+						netconfig->ifindex,
+						netconfig->v4_address,
+						netconfig_ifaddr_del_cmd_cb,
+						netconfig, NULL));
+			l_rtnl_address_free(netconfig->v4_address);
+			netconfig->v4_address = NULL;
+		}
+
+		l_strfreev(netconfig->dns4_overrides);
+		netconfig->dns4_overrides = NULL;
+
+		l_dhcp_client_stop(netconfig->dhcp_client);
+		netconfig->rtm_protocol = 0;
+	}
+}
+
+static void netconfig_ipv4_acd_event(enum l_acd_event event, void *user_data)
+{
+	struct netconfig *netconfig = user_data;
+
+	switch (event) {
+	case L_ACD_EVENT_AVAILABLE:
 		L_WARN_ON(!l_rtnl_ifaddr_add(rtnl, netconfig->ifindex,
 					netconfig->v4_address,
 					netconfig_ipv4_ifaddr_add_cmd_cb,
 					netconfig, NULL));
 		return;
+	case L_ACD_EVENT_CONFLICT:
+		/*
+		 * Conflict found, no IP was actually set so just free/unset
+		 * anything we set prior to starting ACD.
+		 */
+		l_error("netconfig: statically configured address conflict!");
+		l_rtnl_address_free(netconfig->v4_address);
+		netconfig->v4_address = NULL;
+		netconfig->rtm_protocol = 0;
+		break;
+	case L_ACD_EVENT_LOST:
+		/*
+		 * Set IP but lost it some time after. Full (IPv4) reset in this
+		 * case.
+		 */
+		l_error("netconfig: statically configured address was lost");
+		netconfig_reset_v4(netconfig);
+		break;
+	}
+}
+
+static void netconfig_ipv4_select_and_install(struct netconfig *netconfig)
+{
+	netconfig->v4_address = netconfig_get_static4_address(netconfig);
+	if (netconfig->v4_address) {
+		char ip[INET6_ADDRSTRLEN];
+
+		netconfig->rtm_protocol = RTPROT_STATIC;
+		netconfig->acd = l_acd_new(netconfig->ifindex);
+		l_acd_set_event_handler(netconfig->acd,
+					netconfig_ipv4_acd_event, netconfig,
+					NULL);
+		if (getenv("IWD_ACD_DEBUG"))
+			l_acd_set_debug(netconfig->acd, do_debug,
+					"[ACD] ", NULL);
+
+		l_rtnl_address_get_address(netconfig->v4_address, ip);
+
+		if (!l_acd_start(netconfig->acd, ip)) {
+			l_error("failed to start ACD, continuing anyways");
+			l_acd_destroy(netconfig->acd);
+			netconfig->acd = NULL;
+
+			L_WARN_ON(!l_rtnl_ifaddr_add(rtnl, netconfig->ifindex,
+					netconfig->v4_address,
+					netconfig_ipv4_ifaddr_add_cmd_cb,
+					netconfig, NULL));
+		}
+
+		return;
 	}
 
 	netconfig->rtm_protocol = RTPROT_DHCP;
@@ -886,6 +964,8 @@ bool netconfig_configure(struct netconfig *netconfig,
 				const uint8_t *mac_address,
 				netconfig_notify_func_t notify, void *user_data)
 {
+	char *mdns;
+
 	netconfig->dns4_overrides = l_settings_get_string_list(active_settings,
 							"IPv4", "DNS", ' ');
 
@@ -929,6 +1009,11 @@ bool netconfig_configure(struct netconfig *netconfig,
 
 	netconfig_ipv6_select_and_install(netconfig);
 
+	mdns = l_settings_get_string(active_settings,
+						"Network", "MulticastDNS");
+	resolve_set_mdns(netconfig->resolve, mdns);
+	l_free(mdns);
+
 	return true;
 }
 
@@ -952,23 +1037,7 @@ bool netconfig_reset(struct netconfig *netconfig)
 	if (netconfig->rtm_protocol || netconfig->rtm_v6_protocol)
 		resolve_revert(netconfig->resolve);
 
-	if (netconfig->rtm_protocol) {
-		if (netconfig->v4_address) {
-			L_WARN_ON(!l_rtnl_ifaddr_delete(rtnl,
-						netconfig->ifindex,
-						netconfig->v4_address,
-						netconfig_ifaddr_del_cmd_cb,
-						netconfig, NULL));
-			l_rtnl_address_free(netconfig->v4_address);
-			netconfig->v4_address = NULL;
-		}
-
-		l_strfreev(netconfig->dns4_overrides);
-		netconfig->dns4_overrides = NULL;
-
-		l_dhcp_client_stop(netconfig->dhcp_client);
-		netconfig->rtm_protocol = 0;
-	}
+	netconfig_reset_v4(netconfig);
 
 	if (netconfig->rtm_v6_protocol) {
 		l_strfreev(netconfig->dns6_overrides);
