@@ -52,6 +52,7 @@
 #include "src/util.h"
 
 static uint32_t known_networks_watch;
+static uint32_t anqp_watch;
 
 struct network {
 	char ssid[33];
@@ -72,7 +73,10 @@ struct network {
 	bool update_psk:1;  /* Whether PSK should be written to storage */
 	bool ask_passphrase:1; /* Whether we should force-ask agent */
 	bool is_hs20:1;
+	bool anqp_pending:1;	/* Set if there is a pending ANQP request */
 	int rank;
+	/* Holds DBus Connect() message if it comes in before ANQP finishes */
+	struct l_dbus_message *connect_after_anqp;
 };
 
 static bool network_settings_load(struct network *network)
@@ -370,16 +374,18 @@ static int network_load_psk(struct network *network, bool need_passphrase)
 {
 	const char *ssid = network_get_ssid(network);
 	enum security security = network_get_security(network);
-	size_t len;
-	const char *psk = l_settings_get_value(network->settings,
-						"Security", "PreSharedKey");
+	size_t psk_len;
+	uint8_t *psk = l_settings_get_bytes(network->settings, "Security",
+						"PreSharedKey", &psk_len);
 	char *passphrase = l_settings_get_string(network->settings,
 						"Security", "Passphrase");
 	int r;
 
 	/* PSK can be generated from the passphrase but not the other way */
-	if ((!psk || need_passphrase) && !passphrase)
+	if ((!psk || need_passphrase) && !passphrase) {
+		l_free(psk);
 		return -ENOKEY;
+	}
 
 	network_reset_passphrase(network);
 	network_reset_psk(network);
@@ -388,11 +394,12 @@ static int network_load_psk(struct network *network, bool need_passphrase)
 	if (psk) {
 		char *path;
 
-		network->psk = l_util_from_hexstring(psk, &len);
-		if (network->psk && len == 32)
+		if (psk_len == 32) {
+			network->psk = psk;
 			return 0;
+		}
 
-		network_reset_psk(network);
+		l_free(psk);
 
 		path = storage_get_network_file_path(security, ssid);
 		l_error("%s: invalid PreSharedKey format", path);
@@ -435,15 +442,14 @@ void network_sync_psk(struct network *network)
 	fs_settings = storage_network_open(SECURITY_PSK, ssid);
 
 	if (network->psk) {
-		char *hex = l_util_hexstring(network->psk, 32);
-		l_settings_set_value(network->settings, "Security",
-						"PreSharedKey", hex);
+		l_settings_set_bytes(network->settings, "Security",
+						"PreSharedKey",
+						network->psk, 32);
 
 		if (fs_settings)
-			l_settings_set_value(fs_settings, "Security",
-						"PreSharedKey", hex);
-
-		l_free(hex);
+			l_settings_set_bytes(fs_settings, "Security",
+						"PreSharedKey",
+						network->psk, 32);
 	}
 
 	if (network->passphrase) {
@@ -521,6 +527,10 @@ int network_autoconnect(struct network *network, struct scan_bss *bss)
 	bool is_rsn;
 	int ret;
 
+	/* already waiting for an agent request, connect in progress */
+	if (network->agent_request)
+		return -EALREADY;
+
 	switch (security) {
 	case SECURITY_NONE:
 		is_rsn = false;
@@ -552,7 +562,7 @@ int network_autoconnect(struct network *network, struct scan_bss *bss)
 
 	if (!wiphy_select_cipher(wiphy, rsn.pairwise_ciphers) ||
 			!wiphy_select_cipher(wiphy, rsn.group_cipher)) {
-		l_debug("Cipher mis-match");
+		l_debug("Cipher mismatch");
 		ret = -ENETUNREACH;
 		goto close_settings;
 	}
@@ -587,13 +597,13 @@ close_settings:
 	return ret;
 }
 
-void network_connect_failed(struct network *network)
+void network_connect_failed(struct network *network, bool in_handshake)
 {
 	/*
-	 * Connection failed, if PSK try asking for the passphrase
-	 * once more
+	 * Connection failed during the handshake phase.  If PSK try asking
+	 * for the passphrase once more
 	 */
-	if (network_get_security(network) == SECURITY_PSK) {
+	if (network_get_security(network) == SECURITY_PSK && in_handshake) {
 		network->update_psk = false;
 		network->ask_passphrase = true;
 	}
@@ -662,6 +672,27 @@ bool network_bss_add(struct network *network, struct scan_bss *bss)
 	return true;
 }
 
+static bool match_addr(const void *a, const void *b)
+{
+	const struct scan_bss *bss = a;
+
+	return memcmp(bss->addr, b, 6) == 0;
+}
+
+/*
+ * Replaces an old scan_bss (if exists) in the bss list with a new bss object.
+ * Note this BSS is *not* freed and must be by the caller. scan_bss objects are
+ * shared between network/station but station technically owns them.
+ */
+bool network_bss_update(struct network *network, struct scan_bss *bss)
+{
+	l_queue_remove_if(network->bss_list, match_addr, bss->addr);
+
+	l_queue_insert(network->bss_list, bss, scan_bss_rank_compare, NULL);
+
+	return true;
+}
+
 bool network_bss_list_isempty(struct network *network)
 {
 	return l_queue_isempty(network->bss_list);
@@ -673,20 +704,15 @@ void network_bss_list_clear(struct network *network)
 	network->bss_list = l_queue_new();
 }
 
+struct scan_bss *network_bss_list_pop(struct network *network)
+{
+	return l_queue_pop_head(network->bss_list);
+}
+
 struct scan_bss *network_bss_find_by_addr(struct network *network,
 						const uint8_t *addr)
 {
-	const struct l_queue_entry *bss_entry;
-
-	for (bss_entry = l_queue_get_entries(network->bss_list); bss_entry;
-						bss_entry = bss_entry->next) {
-		struct scan_bss *bss = bss_entry->data;
-
-		if (!memcmp(bss->addr, addr, sizeof(bss->addr)))
-			return bss;
-	}
-
-	return NULL;
+	return l_queue_find(network->bss_list, match_addr, addr);
 }
 
 static bool match_bss(const void *a, const void *b)
@@ -892,7 +918,15 @@ static void eap_password_callback(enum agent_result result, const char *value,
 	struct eap_secret_request *req = user_data;
 
 	req->network->agent_request = 0;
-	req->secret->value = l_strdup(value);
+
+	if (value) {
+		if (strlen(value) < IWD_MAX_PASSWORD_LEN)
+			req->secret->value = l_strdup(value);
+		else {
+			l_error("EAP password too long");
+			result = AGENT_RESULT_FAILED;
+		}
+	}
 
 	req->callback(result, message, req);
 }
@@ -910,11 +944,18 @@ static void eap_user_password_callback(enum agent_result result,
 		size_t len1 = strlen(user) + 1;
 		size_t len2 = strlen(passwd) + 1;
 
+		if (len2 > IWD_MAX_PASSWORD_LEN) {
+			l_error("EAP password too long");
+			result = AGENT_RESULT_FAILED;
+			goto done;
+		}
+
 		req->secret->value = l_malloc(len1 + len2);
 		memcpy(req->secret->value, user, len1);
 		memcpy(req->secret->value + len1, passwd, len2);
 	}
 
+done:
 	req->callback(result, message, req);
 }
 
@@ -1123,6 +1164,9 @@ static struct l_dbus_message *network_connect(struct l_dbus *dbus,
 		 */
 		return l_dbus_message_new_method_return(message);
 
+	if (network->agent_request)
+		return dbus_error_busy(message);
+
 	/*
 	 * Select the best BSS to use at this time.  If we have to query the
 	 * agent this may not be the final choice because BSS visibility can
@@ -1141,6 +1185,22 @@ static struct l_dbus_message *network_connect(struct l_dbus *dbus,
 		station_connect_network(station, network, bss, message);
 		return NULL;
 	case SECURITY_8021X:
+		if (network->connect_after_anqp)
+			return dbus_error_busy(message);
+
+		/*
+		 * If there is an ongoing ANQP request we must wait for that to
+		 * finish. Save the message and wait for the ANQP watch to
+		 * fire
+		 */
+		if (network->anqp_pending) {
+			network->connect_after_anqp =
+						l_dbus_message_ref(message);
+			l_debug("Pending ANQP request, delaying connect to %s",
+						network->ssid);
+			return NULL;
+		}
+
 		if (!network_settings_load(network))
 			return dbus_error_not_configured(message);
 
@@ -1150,15 +1210,23 @@ static struct l_dbus_message *network_connect(struct l_dbus *dbus,
 	}
 }
 
-void network_connect_new_hidden_network(struct network *network,
+/*
+ * Returns an error message in case an error occurs.  Otherwise this function
+ * returns NULL and takes a reference to message.  Callers should unref
+ * their copy in this case
+ */
+struct l_dbus_message *network_connect_new_hidden_network(
+						struct network *network,
 						struct l_dbus_message *message)
 {
 	struct station *station = network->station;
 	struct scan_bss *bss;
-	struct l_dbus_message *error;
 
 	l_debug("");
 
+	if (network->agent_request)
+		return dbus_error_busy(message);
+
 	/*
 	 * This is not a Known Network.  If connection succeeds, either
 	 * network_sync_psk or network_connected will save this network
@@ -1166,34 +1234,24 @@ void network_connect_new_hidden_network(struct network *network,
 	 */
 
 	bss = network_bss_select(network, true);
-	if (!bss) {
-		/* This should never happened for the hidden networks. */
-		error = dbus_error_not_supported(message);
-		goto reply_error;
-	}
+	/* This should never happened for the hidden networks. */
+	if (!bss)
+		return dbus_error_not_supported(message);
 
 	network->settings = l_settings_new();
 	l_settings_set_bool(network->settings, "Settings", "Hidden", true);
 
 	switch (network_get_security(network)) {
 	case SECURITY_PSK:
-		error = network_connect_psk(network, bss, message);
-		break;
+		return network_connect_psk(network, bss, message);
 	case SECURITY_NONE:
 		station_connect_network(station, network, bss, message);
-		return;
+		return NULL;
 	default:
-		error = dbus_error_not_supported(message);
 		break;
 	}
 
-	if (error)
-		goto reply_error;
-
-	return;
-
-reply_error:
-	dbus_pending_reply(&message, error);
+	return dbus_error_not_supported(message);
 }
 
 void network_blacklist_add(struct network *network, struct scan_bss *bss)
@@ -1341,7 +1399,7 @@ int network_rank_compare(const void *a, const void *b, void *user)
 	const struct network *new_network = a;
 	const struct network *network = b;
 
-	return network->rank - new_network->rank;
+	return (network->rank > new_network->rank) ? 1 : -1;
 }
 
 void network_rank_update(struct network *network, bool connected)
@@ -1397,22 +1455,30 @@ static void network_unset_hotspot(struct network *network, void *user_data)
 	network_set_info(network, NULL);
 }
 
-static void emit_known_network_changed(struct station *station, void *user_data)
+static void emit_known_network_removed(struct station *station, void *user_data)
 {
 	struct network_info *info = user_data;
-	struct network *network;
+	bool was_hidden = info->is_hidden;
+	struct network *connected_network;
+	struct network *network = NULL;
 
-	if (!info->is_hotspot) {
+	/* Clear network info, as this network is no longer known */
+	if (info->is_hotspot)
+		station_network_foreach(station, network_unset_hotspot, info);
+	else {
 		network = station_network_find(station, info->ssid, info->type);
 		if (!network)
 			return;
 
 		network_set_info(network, NULL);
-		return;
 	}
 
-	/* This is a removed hotspot */
-	station_network_foreach(station, network_unset_hotspot, info);
+	connected_network = station_get_connected_network(station);
+	if (connected_network && connected_network->info == NULL)
+		station_disconnect(station);
+
+	if (network && was_hidden)
+		station_hide_network(station, network);
 }
 
 static void network_update_hotspot(struct network *network, void *user_data)
@@ -1440,17 +1506,6 @@ static void match_known_network(struct station *station, void *user_data)
 	station_network_foreach(station, network_update_hotspot, info);
 }
 
-static void disconnect_no_longer_known(struct station *station, void *user_data)
-{
-	struct network_info *info = user_data;
-	struct network *network;
-
-	network = station_get_connected_network(station);
-
-	if (network && network->info == info)
-		station_disconnect(station);
-}
-
 static void known_networks_changed(enum known_networks_event event,
 					const struct network_info *info,
 					void *user_data)
@@ -1463,12 +1518,40 @@ static void known_networks_changed(enum known_networks_event event,
 		known_network_frequency_sync((struct network_info *)info);
 		break;
 	case KNOWN_NETWORKS_EVENT_REMOVED:
-		station_foreach(disconnect_no_longer_known, (void *) info);
-		station_foreach(emit_known_network_changed, (void *) info);
+		station_foreach(emit_known_network_removed, (void *) info);
 		break;
 	}
 }
 
+static void anqp_watch_changed(enum station_anqp_state state,
+				struct network *network, void *user_data)
+{
+	network->anqp_pending = state == STATION_ANQP_STARTED;
+
+	if (state == STATION_ANQP_FINISHED && network->connect_after_anqp) {
+		struct l_dbus_message *reply;
+
+		l_debug("ANQP complete, resuming connect to %s", network->ssid);
+
+		if (!network_settings_load(network)) {
+			reply = dbus_error_not_configured(
+						network->connect_after_anqp);
+			dbus_pending_reply(&network->connect_after_anqp, reply);
+			return;
+		}
+
+		reply = network_connect_8021x(network,
+					network_bss_select(network, true),
+					network->connect_after_anqp);
+
+		if (reply)
+			l_dbus_send(dbus_get_bus(), reply);
+
+		l_dbus_message_unref(network->connect_after_anqp);
+		network->connect_after_anqp = NULL;
+	}
+}
+
 static void setup_network_interface(struct l_dbus_interface *interface)
 {
 	l_dbus_interface_method(interface, "Connect", 0,
@@ -1502,6 +1585,8 @@ static int network_init(void)
 	known_networks_watch =
 		known_networks_watch_add(known_networks_changed, NULL, NULL);
 
+	anqp_watch = station_add_anqp_watch(anqp_watch_changed, NULL, NULL);
+
 	return 0;
 }
 
@@ -1510,6 +1595,9 @@ static void network_exit(void)
 	known_networks_watch_remove(known_networks_watch);
 	known_networks_watch = 0;
 
+	station_remove_anqp_watch(anqp_watch);
+	anqp_watch = 0;
+
 	l_dbus_unregister_interface(dbus_get_bus(), IWD_NETWORK_INTERFACE);
 }
 
