@@ -29,6 +29,7 @@
 
 #include <ell/ell.h>
 
+#include "ell/useful.h"
 #include "src/module.h"
 #include "src/mpdu.h"
 #include "src/netdev.h"
@@ -388,7 +389,9 @@ static void rrm_handle_beacon_table(struct rrm_state *rrm,
 		l_error("Error reporting beacon table results");
 }
 
-static bool rrm_scan_results(int err, struct l_queue *bss_list, void *userdata)
+static bool rrm_scan_results(int err, struct l_queue *bss_list,
+				const struct scan_freq_set *freqs,
+				void *userdata)
 {
 	struct rrm_state *rrm = userdata;
 	struct rrm_beacon_req_info *beacon = l_container_of(rrm->pending,
@@ -430,7 +433,7 @@ static void rrm_handle_beacon_scan(struct rrm_state *rrm,
 		.freqs = freqs,
 		.flush = true,
 		.duration = beacon->duration,
-		.duration_mandatory = util_is_bit_set(beacon->info.mode, 4),
+		.duration_mandatory = test_bit(&beacon->info.mode, 4),
 	};
 	enum scan_band band = scan_oper_class_to_band(NULL, beacon->oper_class);
 	uint32_t freq;
@@ -476,7 +479,7 @@ static bool rrm_verify_beacon_request(const uint8_t *request, size_t len)
 		 * check for table requests since we just return whatever we
 		 * have cached.
 		 */
-		if (!util_mem_is_zero(request + 2, 2))
+		if (!l_memeqzero(request + 2, 2))
 			return false;
 	}
 
@@ -520,7 +523,7 @@ static void rrm_handle_beacon_request(struct rrm_state *rrm,
 	 * At least for the time being, we will not support autonomous
 	 * reporting, so decline any request to do so.
 	 */
-	if (util_is_bit_set(beacon->info.mode, 1))
+	if (test_bit(&beacon->info.mode, 1))
 		goto reject_refused;
 
 	/*
@@ -530,7 +533,7 @@ static void rrm_handle_beacon_request(struct rrm_state *rrm,
 	 * must reject this request.
 	 */
 	if (!wiphy_has_ext_feature(wiphy, NL80211_EXT_FEATURE_SET_SCAN_DWELL)
-			&& util_is_bit_set(beacon->info.mode, 4))
+			&& test_bit(&beacon->info.mode, 4))
 		goto reject_incapable;
 
 	/* advance to beacon request */
